/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.cdt.core.parser.ast.IASTInclusion;
import org.eclipse.cdt.dstore.core.model.DataElement;

import com.ibm.etools.c.importer.AST.CParseCallback;

/**
 * The CModelConverter class translates the parsed information from the
 * DataStore model to MOF representation.
 * 
 * It finds out all the include files and calls the importSource() method on
 * each file represented by a CSourceUnit object.
 */
public class CModelConverter {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private static CModelConverter INSTANCE;

    //private CImporterResource _importerResource;

    // The C model object list
    private ArrayList _objectList = null;

    // The type table containing all types found in the parse source
    private CTypeTable _typeTable = null;

    // The associated type parser
    private CTypeParser _typeParser = null;

    //  TODO: integration of CDT V2 Parser
    private com.ibm.etools.c.importer.AST.CTypeParser _ASTTypeParser = null;

    // The list of all include files
    private ArrayList _includeList = null;

    private HashMap _sourceUnitTable;

    /**
     * Create an empty CModelConverter
     */
    // public CModelConverter() {
    //   //_importerResource = null;
    //   _objectList = null;
    //   _typeTable = new CTypeTable(this);
    //   _typeParser = new CTypeParser(this);
    //   _includeList = null;
    //   _sourceUnitTable = new HashMap();
    // }
    public static CModelConverter getInstance() {
        return INSTANCE;
    }

    public static CModelConverter createInstance() {
        INSTANCE = new CModelConverter();
        return INSTANCE;
    }

    private CModelConverter() {
        //_importerResource = null;
        _objectList = null;
        _typeTable = new CTypeTable(this);
        _typeParser = new CTypeParser(this);

        // TODO: integration of CDT V2 Parser
        _ASTTypeParser = new com.ibm.etools.c.importer.AST.CTypeParser(this);

        _includeList = null;
        _sourceUnitTable = new HashMap();
    }

    // /**
    //  * Create a CModelConverter from a CImporter
    //  */
    // public CModelConverter(CImporterResource importerResource) {
    //   //_importerResource = importerResource;
    //   _objectList = null;
    //   _typeTable = new CTypeTable(this);
    //   _typeParser = new CTypeParser(this);
    //   _includeList = null;
    //   _sourceUnitTable = new HashMap();
    // }

    // /**
    //  * Return the associated CImporter
    //  */
    // public CImporterResource getImporterResource() {
    //   return _importerResource;
    // }

    /**
     * Return the associated type table
     */
    public CTypeTable getTypeTable() {
        return _typeTable;
    }

    /**
     * Return the associated type parser
     */
    public CTypeParser getTypeParser() {
        return _typeParser;
    }

    /**
     * TODO: integration of CDT V2 Parser
     * 
     * Return the associated type parser
     */
    public com.ibm.etools.c.importer.AST.CTypeParser getASTTypeParser() {
        return _ASTTypeParser;
    }

    /**
     * Return the source unit from a given source file name
     */
    public CSourceUnit getSourceUnitFromName(String fileName) {
        return (CSourceUnit) _sourceUnitTable.get(fileName);
    }

    /**
     * Return the C model object list
     */
    public ArrayList objectList() {
        return _objectList;
    }

    /**
     * Create an array list of C model objects from a DataElement tree. The
     * parsed information is contained in the tree whose root node is the given
     * DataElement.
     * 
     * @return java.util.ArrayList The list of C model objects created from the
     *         parsed information.
     * @param parsed_source
     *            com.ibm.dstore.core.model.DataElement The root node of the
     *            tree under which the parsed information is contained.
     */
    public ArrayList importSource(DataElement parsed_source) throws Exception {

        // Create a new include list for a new import
        _includeList = new ArrayList();

        // Create a new object list for a new import
        _objectList = new ArrayList();

        ArrayList includes = parsed_source.getAssociated(CIM.P_INCLUDES_RELATION);

        if (includes.size() > 0 && CPreferenceStore.isProcessIncludeFiles())
            addSourceWithInlcudes(parsed_source);
        else
            addSource(parsed_source);

        return _objectList;
    }

    /**
     * 
     * TODO: integration of CDT V2 Parser
     * 
     * Create an array list of C model objects from a CParseCallback element.
     * 
     * @param CParseCallback
     *            parsed source
     * 
     * @return java.util.ArrayList The list of C model objects created from the
     *         parsed information.
     * 
     * @exception Exception
     *                if an error occurred
     *  
     */
    public ArrayList importSource(CParseCallback cParseCallback) throws Exception {

        boolean containsInclusions = false;

        // Create a new include list for a new import
        _includeList = new ArrayList();

        // Create a new object list for a new import
        _objectList = new ArrayList();

        Iterator inclusionsIterator = cParseCallback.getInclusions();

        while (inclusionsIterator.hasNext()) {
            containsInclusions = true;
            break;
        }

        if (containsInclusions && CPreferenceStore.isProcessIncludeFiles()) {
            addSourceWithInlcudes(cParseCallback);
        } else {
            addSource(cParseCallback);
        }

        return _objectList;
    }

    /**
     * Add the parsed content of a source file and all of its nested include
     * files to the MOF object list
     */
    private ArrayList addSourceWithInlcudes(DataElement parsed_source) throws Exception {

        ArrayList includes = parsed_source.getAssociated(CIM.P_INCLUDES_RELATION);

        for (int i = 0; i < includes.size(); i++) {
            DataElement includeElement = (DataElement) includes.get(i);
            String includeFileName = includeElement.getName();

            if (!_includeList.contains(includeFileName)) {
                _includeList.add(includeFileName);
                addSourceWithInlcudes(includeElement);
            }
        }

        addSource(parsed_source);
        return _objectList;
    }

    /**
     * 
     * TODO: integration of CDT V2 Parser
     * 
     * Add the parsed content of a source file and all of its nested include
     * files to the object list
     * 
     * @param CParseCallback
     *            parsed source
     * 
     * @return java.util.ArrayList The list of C model objects created from the
     *         parsed information.
     * 
     * @exception Exception
     *                if an error occurred
     */
    private ArrayList addSourceWithInlcudes(CParseCallback cParseCallback) throws Exception {

        Iterator inclusionsIterator = cParseCallback.getInclusions();

        while (inclusionsIterator.hasNext()) {
            IASTInclusion iASTInclusion = (IASTInclusion) inclusionsIterator.next();
            String includeFileName = iASTInclusion.getName();

            if (!_includeList.contains(includeFileName)) {
                _includeList.add(includeFileName);
                //addSourceWithInlcudes(cParseCallback); // TODO: BJAKOB ??
            }
        }

        addSource(cParseCallback);

        return _objectList;
    }

    /**
     * Add the parsed content of a single source file to the object list
     */
    private ArrayList addSource(DataElement source) throws Exception {

        CSourceUnit sourceUnit = (CSourceUnit) _sourceUnitTable.get(source.getName());

        ArrayList srcObjectList = null;
        if (sourceUnit == null) {
            sourceUnit = new CSourceUnit(source, this);
            _sourceUnitTable.put(source.getName(), sourceUnit);
            srcObjectList = sourceUnit.importSource();
        } else {
            srcObjectList = sourceUnit.getObjectList();
        }

        _objectList.addAll(srcObjectList);

        return _objectList;
    }

    /**
     * 
     * TODO: integration of CDT V2 Parser
     * 
     * Add the parsed content of a single source file to the object list
     * 
     * @param CParseCallback
     *            parsed source
     * 
     * @return java.util.ArrayList The list of C model objects created from the
     *         parsed information.
     * 
     * @exception Exception
     *                if an error occurred
     */
    private ArrayList addSource(CParseCallback cParseCallback) throws Exception {

        com.ibm.etools.c.importer.AST.CSourceUnit sourceUnit = (com.ibm.etools.c.importer.AST.CSourceUnit) _sourceUnitTable.get(cParseCallback.getFileName());

        ArrayList srcObjectList = null;

        if (sourceUnit == null) {
            sourceUnit = new com.ibm.etools.c.importer.AST.CSourceUnit(cParseCallback, this);
            _sourceUnitTable.put(cParseCallback.getFileName(), sourceUnit);
            srcObjectList = sourceUnit.importSource();
        } else {
            srcObjectList = sourceUnit.getObjectList();
        }

        _objectList.addAll(srcObjectList);

        return _objectList;
    }

}